/* Ackermann.java
 *
 * The Ackermann function is defined recursively for non-negative integers m and n as follows:
 * A(m, n) = n + 1                 if m = 0
 * A(m, n) = A(m - 1, 1)           if m > 0 and n = 0
 * A(m, n) = A(m - 1, A(m, n - 1)) if m > 0 and n > 0
 * 
 * 2009 yz
 */

public class Ackermann {
    private static final int M = 1000;
    
    public static int akmRecursive(int m, int n) {
	if(m == 0)
	    return (n + 1);
	if(m > 0 && n == 0)
	    return akmRecursive(m - 1, 1);
	if(m > 0 && n > 0)
	    return akmRecursive(m - 1, akmRecursive(m, n - 1));
	
	return -1;
    }
    
    public static int akmNonrecursive(int m, int n) {
	int i = 1;
	int[] s = new int[M];
	
	s[0] = m;
	s[1] = n;
	
	while(i > 0) {
	    n = s[i--];
	    m = s[i];
	    
	    if(m == 0)
		s[i] = n + 1;
	    
	    if(m > 0 && n == 0) {
		s[i++] = m - 1;
		s[i] = 1;
	    }
	    
	    if(m > 0 && n > 0) {
		s[i++] = m - 1;
		s[i++] = m;
		s[i] = n - 1;
	    }
	}
	
	return s[0];
    }
    
    public static void main(String[] args) {
	System.out.println("akmRecursive(3, 4) = " + akmRecursive(3, 4));
	System.out.println("akmNonrecursive(3, 4) = " + akmNonrecursive(3, 4));
    }
}
